/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::LaserScannerPluginWidgetController
 * @author     Fabian Paus ( fabian dot paus at kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_RobotAPI_LaserScannerPlugin_WidgetController_H
#define _ARMARX_RobotAPI_LaserScannerPlugin_WidgetController_H

#include "ui_LaserScannerPluginWidget.h"

#include <ArmarXCore/core/system/ImportExportComponent.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <ArmarXGui/libraries/SimpleConfigDialog/SimpleConfigDialog.h>
#include <RobotAPI/interface/units/LaserScannerUnit.h>

namespace armarx
{
    /**
    \page ArmarXGui-GuiPlugins-LaserScannerPlugin LaserScannerPlugin
    \brief The LaserScannerPlugin allows visualizing ...

    \image html LaserScannerPlugin.png
    The user can

    API Documentation \ref LaserScannerPluginWidgetController

    \see LaserScannerPluginGuiPlugin
    */

    /**
     * \class LaserScannerPluginWidgetController
     * \brief LaserScannerPluginWidgetController brief one line description
     *
     * Detailed description
     */
    class ARMARXCOMPONENT_IMPORT_EXPORT
        LaserScannerPluginWidgetController:
    public armarx::ArmarXComponentWidgetController,
    public armarx::LaserScannerUnitListener
    {
        Q_OBJECT

    public:
        /**
         * Controller Constructor
         */
        explicit LaserScannerPluginWidgetController();

        /**
         * Controller destructor
         */
        virtual ~LaserScannerPluginWidgetController();

        /**
         * @see ArmarXWidgetController::loadSettings()
         */
        virtual void loadSettings(QSettings* settings);

        /**
         * @see ArmarXWidgetController::saveSettings()
         */
        virtual void saveSettings(QSettings* settings);

        /**
         * Returns the Widget name displayed in the ArmarXGui to create an
         * instance of this class.
         */
        virtual QString getWidgetName() const
        {
            return "LaserScannerPlugin";
        }

        /**
         * \see armarx::Component::onInitComponent()
         */
        virtual void onInitComponent();

        /**
         * \see armarx::Component::onConnectComponent()
         */
        virtual void onConnectComponent();

        QPointer<QDialog> getConfigDialog(QWidget *parent) override;

        void configured() override;

        void reportSensorValues(const std::string & device, const std::string & name,
                                const LaserScan & scan, const TimestampBasePtr & timestamp,
                                const Ice::Current & c) override;
    public slots:
        void onNewSensorValuesReported();

    signals:
        void newSensorValuesReported();

    private:
        /**
         * Widget Form
         */
        Ui::LaserScannerPluginWidget widget;
        QPointer<SimpleConfigDialog> dialog;

        std::string laserScannerUnitName;
        LaserScannerUnitInterfacePrx laserScannerUnit;

        Mutex scanMutex;
        std::unordered_map<std::string, LaserScan> scans;

        QGraphicsScene scene;
    };
}

#endif
